import React, { useEffect, useState } from 'react';
import * as THREE from 'three';
import { OrbitControls } from 'three/examples/jsm/controls/OrbitControls.js';
import { FBXLoader } from 'three/examples/jsm/loaders/FBXLoader.js';
import { OBJLoader } from 'three/examples/jsm/loaders/OBJLoader.js';
import { MTLLoader } from 'three/examples/jsm/loaders/MTLLoader.js';
import { TransformControls } from 'three/examples/jsm/controls/TransformControls.js';

const Admin = () => {
  let scene = null,
    camera = null,
    renderer = null,
    controls = null,
    container = null,
    transform = null,
    mouse = new THREE.Vector3(),
    meshs = [],
    raycaster = new THREE.Raycaster();
  let onlyModel;
  let aspect = window.innerWidth / window.innerHeight;
  const [Img, setImg] = useState(
    '/c4d13e1ef5c20e49d7206aefff123f19--no-signa.jpg'
  );
  const [errImg, setErrImg] = useState('');
  // const [fbxModel, setfbxModel] = useState('/horseMorpher updated.FBX');
  const [errFbxModel, setErrFbxModel] = useState('');
  const [scaleX, setScaleX] = useState(0.01);
  const [scaleY, setScaleY] = useState(0.01);
  const [scaleZ, setScaleZ] = useState(0.01);
  const [posX, setPosX] = useState(0);
  const [posY, setPosY] = useState(0);
  const [posZ, setPosZ] = useState(0);
  const [rotateX, setRotateX] = useState(0);
  const [rotateY, setRotateY] = useState(0);
  const [rotateZ, setRotateZ] = useState(0);

  const [modalMarker, setModalMarker] = useState(false);
  const [model3d, setModel3d] = useState(false);
  const [isTranslate, setIsTranslate] = useState(true);
  const [isRotate, setIsRotate] = useState(false);
  const [isScale, setIsScale] = useState(false);

  // const [activeObjectNameUuid, setActiveObjectNameUuid] = useState({});
  const [uploadFiles, setUploadFiles] = useState({
    arrOfTexture: [],
    mtl: { name: '', url: '' },
    url: { name: '', url: '' },
  });

  useEffect(() => {
    init();
  }, [
    Img,
    // fbxModel,
    scaleX,
    scaleY,
    scaleZ,
    posX,
    posY,
    posZ,
    rotateX,
    rotateY,
    rotateZ,
    modalMarker,
    isTranslate,
    isRotate,
    isScale,
    meshs,
    uploadFiles,
  ]);

  // useEffect(() => {
  //   return function cleanUp() {
  //     container.removeChild(renderer.domElement);
  //   };
  // });

  // useEffect(() => {
  //   return function cleanUp() {
  //     transform.removeEventListener('change', renderer.domElement);
  //     transform.removeEventListener('dragging-changed', () => {
  //       changeDragPropertiesValue();
  //       controls.enabled = false;
  //     });
  //     transform.detach();
  //   };
  // });

  const init = () => {
    LoadRest();
    LoadScene();
    LoadCamera();
    LoadPointLight();
    LoadAmbientLight();
    LoadRenderer();
    LoadControl();
    LoadFloor();
    LoadModel();
    LoadTransformControl();
    !Img && LoadGrid();

    animate();
  };

  const LoadRest = () => {
    scene = null;
    camera = null;
    renderer = null;
    controls = null;
    transform = null;
  };

  const ResetTransform = () => {
    transform.removeEventListener('change', renderer.domElement);
    transform.removeEventListener('dragging-changed', (event) => {
      controls.enabled = false;
    });
    transform.detach();
  };

  const changeDragPropertiesValue = () => {};

  const LoadScene = () => {
    scene = new THREE.Scene();
    scene.background = new THREE.Color(0xa9a9a9);
  };

  const LoadCamera = () => {
    camera = new THREE.PerspectiveCamera(75, aspect, 1, 1000); //fov,aspect,in,out
    camera.position.x = 0;
    camera.position.y = 4;
    camera.position.z = 5;
    scene.add(camera);
  };

  const LoadPointLight = () => {
    let light = new THREE.PointLight();
    light.intensity = 3;
    light.position.set(0, 2, 3);
    scene.add(light);
  };

  const LoadAmbientLight = () => {
    let ambientLight = new THREE.AmbientLight();
    scene.add(ambientLight);
  };

  const LoadRenderer = () => {
    renderer = new THREE.WebGLRenderer({ antialias: true });
    renderer.setSize(window.innerWidth / 1.6, window.innerHeight / 1.45);
    document.body.appendChild(renderer.domElement);
    container = document.querySelector('#Canvas');
    if (container && container.childElementCount > 0) {
      container.innerHTML = '';
    }
    container.appendChild(renderer.domElement);

    raycaster = new THREE.Raycaster();
    mouse = new THREE.Vector3();
    document.addEventListener('click', mouseUpHandler, false);

    window.addEventListener('resize', () => {
      const width = window.innerWidth;
      const height = window.innerHeight;
      renderer.setSize(width / 1.6, height / 1.45);
    });
  };

  const mouseUpHandler = (event, touch = false) => {
    event.preventDefault();
    let rect = renderer.domElement.getBoundingClientRect();
    mouse.x = ((event.clientX - rect.left) / (rect.width - rect.left)) * 2 - 1;
    mouse.y = -((event.clientY - rect.top) / (rect.bottom - rect.top)) * 2 + 1;

    let vector = new THREE.Vector3(mouse.x, mouse.y, 0.5);
    vector = vector.unproject(camera);
    const raycaster = new THREE.Raycaster(
      camera.position,
      vector.sub(camera.position).normalize()
    );
    raycaster.setFromCamera(mouse, camera);

    const hits = raycaster.intersectObjects(onlyModel, true);
    console.log('hits', hits);
    if (hits[0]) {
      const model = hits[0].userData.root;
      console.log('model', model);
      // Do something with the root model.
    }

    const intersects = raycaster.intersectObjects(scene.children);
    for (let i = 0; i < intersects.length; i++) {
      console.log('intersects', intersects);
      intersects[i].object.material.color.set(0xff0000);
      console.log('intersects[i].object', intersects[i].object.position.x);
      console.log('intersects[i].object', intersects[i].object.position.y);
      console.log('intersects[i].object', intersects[i].object.position.z);
      setPosX(intersects[i].object.position.x);
      setPosY(intersects[i].object.position.y);
      setPosZ(intersects[i].object.position.z);
    }
  };
  // distance – distance between the origin of the ray and the intersection
  // point – point of intersection, in world coordinates
  // face – intersected face
  // faceIndex – index of the intersected face
  // object – the intersected object
  // uv - U,V coordinates at point of intersection
  // uv2 - Second set of U,V coordinates at point of intersection
  // instanceId – The index number of the instance where the ray intersects the InstancedMesh

  const LoadControl = () => {
    controls = new OrbitControls(camera, renderer.domElement);
    controls.update();
    controls.addEventListener('change', render);
  };

  const LoadGrid = () => {
    let size = 8;
    let boxes = 8;
    let gridControl = new THREE.GridHelper(size, boxes);
    gridControl.rotation.x = Math.PI / 3.6;

    scene.add(gridControl);
  };

  const LoadTransformControl = () => {
    transform = new TransformControls(camera, renderer.domElement);
    transform.addEventListener('change', () => renderer.render(scene, camera));
    transform.addEventListener('dragging-changed', (event) => {
      changeDragPropertiesValue();
      transform.setSpace('local');
      controls.enabled = !event.value;
    });

    scene.add(transform);
  };

  const LoadFloor = () => {
    const textureLoader = new THREE.TextureLoader();
    const t1 = textureLoader.load(Img);
    const groundGeometry = new THREE.PlaneBufferGeometry(20, 20, 10, 10);
    const groundMaterial = new THREE.MeshBasicMaterial({
      map: t1,
    });
    const ground = new THREE.Mesh(groundGeometry, groundMaterial);
    ground.rotation.x = -Math.PI / 4.5;
    ground.rotation.y = 0;
    ground.rotation.z = 0;
    ground.scale.x = 0.4;
    ground.scale.y = 0.4;
    ground.scale.z = 0.4;
    ground.name = 'Ground';
    ground.geometry.computeBoundingBox();
    ground.geometry.needsUpdate = true;

    scene.add(ground);
    meshs.push(ground);
  };

  const LoadModel = () => {
    const loadedMaterials = [];
    const texture = new THREE.Texture();
    const manager = new THREE.LoadingManager();

    manager.onStart = function (url, itemsLoaded, itemsTotal) {
      console.log(
        'Started loading file: ' +
          url +
          '.\nLoaded ' +
          itemsLoaded +
          ' of ' +
          itemsTotal +
          ' files.'
      );
    };
    manager.onLoad = () => {
      if (loadedMaterials.length > 0) {
        const objLoader = new OBJLoader();
        objLoader.setMaterials(loadedMaterials[0]);
        objLoader.load(
          '/Lamborginhi Aventador OBJ/Lamborghini_Aventador.obj',
          (object) => {
            object.traverse((child) => {
              if (child instanceof THREE.Mesh) {
                child.material =
                  child.material || new THREE.MeshPhongMaterial();
                child.material['map'] = texture;
              }
            });
            object.receiveShadow = true;
            object.castShadow = true;
            object.name = 'modelObj.content.name';
            object.matrixAutoUpdate = true;
            object.scale.x = scaleX;
            object.scale.y = scaleY;
            object.scale.z = scaleZ;
            object.position.x = posX;
            object.position.y = posY;
            object.position.z = posZ;
            object.rotation.x = rotateX;
            object.rotation.y = rotateY;
            object.rotation.z = rotateZ;
            object.updateMatrix();
            scene.add(object);
            transform.attach(object);
          }
        );
      }
    };

    manager.onProgress = function (url, itemsLoaded, itemsTotal) {
      console.log(
        'Loading file: ' +
          url +
          '.\nLoaded ' +
          itemsLoaded +
          ' of ' +
          itemsTotal +
          ' files.'
      );
    };

    manager.onError = function (url) {
      console.log('There was an error loading ' + url);
    };

    const loader = new THREE.ImageLoader(manager);
    const arrOfTexture = [
      '/Lamborginhi Aventador OBJ/Lamborginhi Aventador_diffuse.jpeg',
      '/Lamborginhi Aventador OBJ/Lamborginhi Aventador_gloss.jpeg',
      '/Lamborginhi Aventador OBJ/Lamborginhi Aventador_spec.jpeg',
    ];
    for (let i = 0, length = arrOfTexture.length; i < length; i++) {
      loader.load(arrOfTexture[i], (img) => {
        console.log('texture');
        if (img) {
          texture.image = img;
          texture.mapping = THREE.UVMapping;
          texture.needsUpdate = true;
        }
      });
    }
    // for multiple mtl files loader
    const mtlLoader = new MTLLoader(manager);
    mtlLoader.load(
      '/Lamborginhi Aventador OBJ/Lamborghini_Aventador.mtl',
      (materials) => {
        console.log('MTLLoader');
        materials.preload();
        loadedMaterials.push(materials);
      }
    );
  };

  const animate = () => {
    requestAnimationFrame(animate);
    scene.updateMatrixWorld();
    render();
  };

  const render = () => {
    renderer.render(scene, camera);
  };

  const selectContolButton = (selectedPropertyTypes) => {
    // console.log('selectedPropertyTypes', selectedPropertyTypes);
    if (selectedPropertyTypes === 'isTranslate') {
      setIsTranslate(true);
      setIsRotate(false);
      setIsScale(false);
      // console.log('transform.setMode', transform.setMode);
      transform.setMode('translate');
    }
    if (selectedPropertyTypes === 'isRotate') {
      setIsTranslate(false);
      setIsRotate(true);
      setIsScale(false);
      transform.setMode('rotate');
    }
    if (selectedPropertyTypes === 'isScale') {
      setIsTranslate(false);
      setIsRotate(false);
      setIsScale(true);
      transform.setMode('scale');
    }
  };

  // console.log('uploadFiles', uploadFiles);
  return (
    <>
      <div className="main-box">
        <div className="row">
          <div className="col-8">
            <div
              style={{
                border: '1px grey solid',
                padding: '10px',
                marginTop: '5px',
              }}
            >
              <div id="Canvas" />

              <div className="editor-btn">
                <div>scaleX : {scaleX} </div>
                <div>scaleY : {scaleY} </div>
                <div>scaleZ : {scaleZ} </div>
                <div>posX, : {posX} </div>
                <div>posY, : {posY} </div>
                <div>posZ, : {posZ} </div>
                <div>rotatex : {rotateX} </div>
                <div>rotatey : {rotateY} </div>
                <div>rotatez : {rotateZ} </div>
              </div>
            </div>
          </div>
        </div>
      </div>
    </>
  );
};

export default Admin;
