/* eslint-disable camelcase */
import axios from 'axios';

// axios.defaults.baseURL = process.env.REACT_APP_API_PATH;

/**
 * Global request function for handling all HTTP calls
 * @param method
 * @param url
 * @param options {params: <for query params>, payload: <for data to be sent>'}
 */

const request = (
  method,
  url,
  options = { params: {}, payload: {}, requestOptions: {} }
) => {
  const request = {
    url,
    method,
    params: options.params,
    data: options.payload,
    ...options.requestOptions,
  };

  return new Promise((resolve, reject) => {
    axios
      .request(request)
      .then((res) => resolve(res))
      .catch((err) => reject(err));
  });
};

export const POST = (path, payload, requestOptions) => {
  return request('POST', path, { payload, requestOptions });
};

export const GET = (path, params) => {
  return request('GET', path, { params });
};

export const PUT = (path, payload) => {
  return request('PUT', path, { payload });
};

export const PATCH = (path, payload) => {
  return request('PATCH', path, { payload });
};

export const DELETE = (path) => {
  return request('DELETE', path);
};
